@@ -6,18 +6,15 @@ Plot of the canonical Glover HRF
 
 import numpy as np
 
-from nipy.modalities.fmri import hrf
+from nipy.modalities.fmri import hrf, utils
 
-import pylab
+import matplotlib.pyplot as plt
 
-
-from matplotlib import rc
-rc('text', usetex=True)
+# hrf.glover is a symbolic function; get a function of time to work on arrays
+hrf_func = utils.lambdify_t(hrf.glover(utils.T))
 
 t = np.linspace(0,25,200)
-pylab.plot(t, hrf.glover(t))
-a=pylab.gca()
+plt.plot(t, hrf_func(t))
+a=plt.gca()
 a.set_xlabel(r'$t$')
 a.set_ylabel(r'$h_{can}(t)$')
-
-
