@@ -1,21 +1,27 @@
+#!/usr/bin/env python
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-"""
-This scipt generates a noisy activation image image
-and extracts the blobs from it.
+from __future__ import print_function # Python 2/3 compatibility
+__doc__ = """
+This scipt makes a noisy activation image and extracts the blobs from it.
 
-Author : Bertrand Thirion, 2009
+Requires matplotlib
+
+Author : Bertrand Thirion, 2009--2012
 """
-#autoindent
-print __doc__
+print(__doc__)
 
 import numpy as np
-import pylab as pl
-import matplotlib
+
+try:
+    import matplotlib.pyplot as plt
+except ImportError:
+    raise RuntimeError("This script needs the matplotlib library")
+import matplotlib as mpl
 
 import nipy.labs.utils.simul_multisubject_fmri_dataset as simul
 import nipy.labs.spatial_models.hroi as hroi
-from nipy.labs.spatial_models.discrete_domain import domain_from_binary_array
+from nipy.labs.spatial_models.discrete_domain import grid_domain_from_shape
 
 # ---------------------------------------------------------
 # simulate an activation image
@@ -28,23 +34,19 @@ dataset = simul.surrogate_2d_dataset(n_subj=1, shape=shape, pos=pos,
                                      ampli=ampli, width=10.0).squeeze()
 values = dataset.ravel()
 
-
 #-------------------------------------------------------
 # Computations
 #-------------------------------------------------------
 
 # create a domain descriptor associated with this
-domain = domain_from_binary_array(dataset ** 2 > 0)
+domain = grid_domain_from_shape(shape)
+nroi = hroi.HROI_as_discrete_domain_blobs(domain, values, threshold=2.0, smin=3)
 
-nroi = hroi.HROI_as_discrete_domain_blobs(domain, dataset.ravel(),
-                                          threshold=2.0, smin=3)
 # create an average activaion image
-flat_dataset = dataset.ravel()
-activation = [flat_dataset[nroi.select_id(id, roi=False)]
-              for id in nroi.get_id()]
+activation = [values[nroi.select_id(id, roi=False)] for id in nroi.get_id()]
 nroi.set_feature('activation', activation)
 bmap = nroi.feature_to_voxel_map(
-    'activation', roi=True, method="mean").reshape((dimx, dimy))
+    'activation', roi=True, method="mean").reshape(shape)
 
 #--------------------------------------------------------
 # Result display
@@ -64,37 +66,37 @@ cdict = {'red': ((0.0, 0.0, 0.7),
                   (aux1, 0.7, 0.0),
                   (aux2, 0.5, 0.5),
                   (1.0, 1.0, 1.0))}
-my_cmap = matplotlib.colors.LinearSegmentedColormap('my_colormap', cdict, 256)
+my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap', cdict, 256)
 
-pl.figure(figsize=(12, 3))
-pl.subplot(1, 3, 1)
-pl.imshow(dataset, interpolation='nearest', cmap=my_cmap)
-cb = pl.colorbar()
+plt.figure(figsize=(12, 3))
+plt.subplot(1, 3, 1)
+plt.imshow(dataset, interpolation='nearest', cmap=my_cmap)
+cb = plt.colorbar()
 for t in cb.ax.get_yticklabels():
     t.set_fontsize(16)
 
-pl.axis('off')
-pl.title('Thresholded data')
+plt.axis('off')
+plt.title('Thresholded data')
 
 # plot the blob label image
-pl.subplot(1, 3, 2)
-pl.imshow(nroi.feature_to_voxel_map('id', roi=True).reshape((dimx, dimy)),
+plt.subplot(1, 3, 2)
+plt.imshow(nroi.feature_to_voxel_map('id', roi=True).reshape(shape),
           interpolation='nearest')
-pl.colorbar()
-pl.title('Blob labels')
+plt.colorbar()
+plt.title('Blob labels')
 
 # plot the blob-averaged signal image
 aux = 0.01
 cdict = {'red': ((0.0, 0.0, 0.7), (aux, 0.7, 0.7), (1.0, 1.0, 1.0)),
          'green': ((0.0, 0.0, 0.7), (aux, 0.7, 0.0), (1.0, 1.0, 1.0)),
          'blue': ((0.0, 0.0, 0.7), (aux, 0.7, 0.0), (1.0, 0.5, 1.0))}
-my_cmap = matplotlib.colors.LinearSegmentedColormap('my_colormap', cdict, 256)
+my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap', cdict, 256)
 
-pl.subplot(1, 3, 3)
-pl.imshow(bmap, interpolation='nearest', cmap=my_cmap)
-cb = pl.colorbar()
+plt.subplot(1, 3, 3)
+plt.imshow(bmap, interpolation='nearest', cmap=my_cmap)
+cb = plt.colorbar()
 for t in cb.ax.get_yticklabels():
     t.set_fontsize(16)
-pl.axis('off')
-pl.title('Blob average')
-pl.show()
+plt.axis('off')
+plt.title('Blob average')
+plt.show()
