@@ -2,10 +2,17 @@
 Infinite mixture model : A generalization of Bayesian mixture models
 with an unspecified number of classes
 """
+from __future__ import print_function
+from __future__ import absolute_import
+from __future__ import division
+
+import math
+
 import numpy as np
-from bgmm import BGMM, detsh
+
 from scipy.special import gammaln
 
+from .bgmm import BGMM, detsh
 
 def co_labelling(z, kmax=None, kmin=None):
     """
@@ -28,10 +35,10 @@ def co_labelling(z, kmax=None, kmin=None):
     n = z.size
     colabel = coo_matrix((n, n))
 
-    if kmax == None:
+    if kmax is None:
         kmax = z.max() + 1
 
-    if kmin == None:
+    if kmin is None:
         kmin = z.min() - 1
 
     for  k in np.unique(z):
@@ -142,7 +149,7 @@ class IMM(BGMM):
                     total likelihood of the model
         """
         self.check_x(x)
-        if sampling_points == None:
+        if sampling_points is None:
             average_like = np.zeros(x.shape[0])
         else:
             average_like = np.zeros(sampling_points.shape[0])
@@ -159,12 +166,12 @@ class IMM(BGMM):
         z = self.sample_indicator(like)
 
         for i in range(niter):
-            if  kfold == None:
+            if  kfold is None:
                 like = self.simple_update(x, z, plike)
             else:
                 like = self.cross_validated_update(x, z, plike, kfold)
 
-            if sampling_points == None:
+            if sampling_points is None:
                 average_like += like
             else:
                 average_like += np.sum(
@@ -230,7 +237,7 @@ class IMM(BGMM):
         if np.isscalar(kfold):
             aux = np.argsort(np.random.rand(n_samples))
             idx = - np.ones(n_samples).astype(np.int)
-            j = np.ceil(n_samples / kfold)
+            j = int(math.ceil(n_samples / kfold))
             kmax = kfold
             for k in range(kmax):
                 idx[aux[k * j:min(n_samples, j * (k + 1))]] = k
@@ -332,10 +339,10 @@ class IMM(BGMM):
         -------
         z: array of shape(nbitem): a draw of the membership variable
 
-        Note
-        ----
-        The behaviour is different from standard bgmm
-        in that z can take arbitrary values
+        Notes
+        -----
+        The behaviour is different from standard bgmm in that z can take
+        arbitrary values
         """
         z = BGMM.sample_indicator(self, like)
         z[z == self.k] = self.k + np.arange(np.sum(z == self.k))
@@ -393,7 +400,7 @@ class IMM(BGMM):
         like, array of shape(nbitem,self.k)
         component-wise likelihood
         """
-        if plike == None:
+        if plike is None:
             plike = self.likelihood_under_the_prior(x)
 
         plike = np.reshape(plike, (x.shape[0], 1))
@@ -481,7 +488,7 @@ class MixedIMM(IMM):
         self.check_x(x)
         pproba = np.zeros(x.shape[0])
 
-        if sampling_points == None:
+        if sampling_points is None:
             average_like = np.zeros(x.shape[0])
         else:
             average_like = np.zeros(sampling_points.shape[0])
@@ -502,7 +509,7 @@ class MixedIMM(IMM):
             coclust = coo_matrix((x.shape[0], x.shape[0]))
 
         for i in range(niter):
-            if  kfold == None:
+            if  kfold is None:
                 like = self.simple_update(x, z, plike, null_class_proba)
             else:
                 like, z = self.cross_validated_update(x, z, plike,
@@ -515,7 +522,7 @@ class MixedIMM(IMM):
             if co_clustering:
                 coclust = coclust + co_labelling(z, self.k, -1)
 
-            if sampling_points == None:
+            if sampling_points is None:
                 average_like += like
             else:
                 average_like += np.sum(
@@ -584,10 +591,10 @@ class MixedIMM(IMM):
         z: array of shape(n_samples),
               the associated membership variables
 
-        Note
-        ----
-        when kfold is an array, there is an internal reshuffling
-        to randomize the order of updates
+        Notes
+        -----
+        When kfold is an array, there is an internal reshuffling to randomize
+        the order of updates
         """
         n_samples = x.shape[0]
         slike = np.zeros(n_samples)
@@ -595,7 +602,7 @@ class MixedIMM(IMM):
         if np.isscalar(kfold):
             aux = np.argsort(np.random.rand(n_samples))
             idx = - np.ones(n_samples).astype(np.int)
-            j = np.ceil(n_samples / kfold)
+            j = int(math.ceil(n_samples / kfold))
             kmax = kfold
             for k in range(kmax):
                 idx[aux[k * j:min(n_samples, j * (k + 1))]] = k
@@ -646,8 +653,8 @@ class MixedIMM(IMM):
         -------
         z: array of shape(nbitem): a draw of the membership variable
 
-        Note
-        ----
+        Notes
+        -----
         Here z=-1 encodes for the null class
         """
         n = like.shape[0]
@@ -673,13 +680,13 @@ def main():
 
     # warming
     igmm.sample(x, niter=100, kfold=10)
-    print 'number of components: ', igmm.k
+    print('number of components: ', igmm.k)
 
     #
-    print 'number of components: ', igmm.k
+    print('number of components: ', igmm.k)
 
     if dim < 3:
-        from gmm import plot2D
+        from .gmm import plot2D
         plot2D(x, igmm, verbose=1)
     return igmm
 
