@@ -3,6 +3,7 @@
 """
 Linear filter(s).  For the moment, only a Gaussian smoothing filter
 """
+from __future__ import absolute_import
 
 import gc
 
@@ -10,6 +11,7 @@ import numpy as np
 import numpy.fft as fft
 import numpy.linalg as npl
 
+from nipy.utils import seq_prod
 from nipy.core.api import Image, AffineTransform
 from nipy.core.reference.coordinate_map import product
 
@@ -58,10 +60,10 @@ class LinearFilter(object):
         # reshape to (N coordinates, -1).  We appear to need to assign
         # to shape instead of doing a reshape, in order to avoid memory
         # copies
-        voxels.shape = (voxels.shape[0], np.product(voxels.shape[1:]))
+        voxels.shape = (voxels.shape[0], seq_prod(voxels.shape[1:]))
         # physical coordinates relative to center
         X = (self.coordmap(voxels.T) - phys_center).T
-        X.shape = (self.coordmap.ndims[0],) + tuple(self.bshape)
+        X.shape = (self.coordmap.ndims[1],) + tuple(self.bshape)
         # compute kernel from these positions
         kernel = self(X, axis=0)
         kernel = _crop(kernel)
@@ -69,8 +71,9 @@ class LinearFilter(object):
                       'l1':np.fabs(kernel).sum(),
                       'l1sum':kernel.sum()}
         self._kernel = kernel
-        self.shape = (np.ceil((np.asarray(self.bshape) +
-                              np.asarray(kernel.shape))/2)*2+2)
+        self.shape = (np.ceil(
+            (np.asarray(self.bshape) + np.asarray(kernel.shape)) / 2)
+            * 2 + 2).astype(np.intp)
         self.fkernel = np.zeros(self.shape)
         slices = [slice(0, kernel.shape[i]) for i in range(len(kernel.shape))]
         self.fkernel[slices] = kernel
@@ -101,7 +104,7 @@ class LinearFilter(object):
             for i in range(len(self.bshape)):
                 _X[i] /= f[i]
         # whiten?
-        if self.cov != None:
+        if self.cov is not None:
             _chol = npl.cholesky(self.cov)
             _X = np.dot(npl.inv(_chol), _X)
         # compute squared distance
@@ -178,8 +181,9 @@ class LinearFilter(object):
                 _out = data
             _slice += 1
         gc.collect()
-        _out = _out[[slice(self._kernel.shape[i]/2, self.bshape[i] +
-                           self._kernel.shape[i]/2) for i in range(len(self.bshape))]]
+        _out = _out[[slice(self._kernel.shape[i] // 2,
+                           self.bshape[i] + self._kernel.shape[i] // 2)
+                     for i in range(len(self.bshape))]]
         if inimage.ndim == 3:
             return Image(_out, coordmap=self.coordmap)
         else:
