@@ -21,6 +21,8 @@ It implements 3 approaches:
 
 Author : Bertrand Thirion, Yaroslav Halchenko, 2008-2012
 """
+from __future__ import print_function
+from __future__ import absolute_import
 
 import numpy as np
 from numpy.linalg import pinv
@@ -190,9 +192,9 @@ class NormalEmpiricalNull(object):
 
         * mu = mu
         * p0 = min(1, np.exp(lp0))
-        * sqsigma: standard deviation of the estimated normal
+        * sqsigma: variance of the estimated normal
           distribution
-        * sigma: np.sqrt(sqsigma) : variance of the estimated
+        * sigma: np.sqrt(sqsigma) : standard deviation of the estimated
           normal distribution
         """
         # take a central subsample of x
@@ -200,23 +202,22 @@ class NormalEmpiricalNull(object):
 
         # generate the histogram
         step = 3.5 * np.std(self.x) / np.exp(np.log(self.n) / 3)
-        bins = max(10, (self.x.max() - self.x.min()) / step)
+        bins = max(10, int((self.x.max() - self.x.min()) // step))
         hist, ledge = np.histogram(x, bins=bins)
         step = ledge[1] - ledge[0]
         medge = ledge + 0.5 * step
 
         # remove null bins
-        whist = hist > 0
-        hist = hist[whist]
-        medge = medge[whist]
-        hist = hist.astype('f')
+        hist = hist[hist > 0].astype(np.float)
+        medge = medge[:-1][hist > 0]  # edges include rightmost outer
 
         # fit the histogram
-        dmtx = np.ones((3, np.sum(whist)))
+        dmtx = np.ones((3, len(hist)))
         dmtx[1] = medge
         dmtx[2] = medge ** 2
         coef = np.dot(np.log(hist), pinv(dmtx))
-        sqsigma = -1.0 / (2 * coef[2])
+        sqsigma = - 1.0 / (2 * coef[2])
+        sqsigma = max(sqsigma, 1.e-6)
         mu = coef[1] * sqsigma
         lp0 = (coef[0] - np.log(step * self.n)
                 + 0.5 * np.log(2 * np.pi * sqsigma) + mu ** 2 / (2 * sqsigma))
@@ -264,8 +265,8 @@ class NormalEmpiricalNull(object):
             self.plot(efp, alpha)
 
         if efp[-1] > alpha:
-            print "the maximal value is %f , the corresponding FDR is %f " \
-                    % (self.x[ - 1], efp[ - 1])
+            print("the maximal value is %f , the corresponding FDR is %f "
+                  % (self.x[ - 1], efp[ - 1]))
             return np.inf
         j = np.argmin(efp[:: - 1] < alpha) + 1
         return 0.5 * (self.x[ - j] + self.x[ - j + 1])
@@ -357,7 +358,7 @@ class NormalEmpiricalNull(object):
         hist /= step
 
         import matplotlib.pylab as mp
-        if mpaxes == None:
+        if mpaxes is None:
             mp.figure()
             ax = mp.subplot(1, 1, 1)
         else:
@@ -376,7 +377,7 @@ class NormalEmpiricalNull(object):
         ax.set_xticklabels(ax.get_xticks(), fontsize=12)
         ax.set_yticklabels(ax.get_yticks(), fontsize=12)
 
-        if efp != None:
+        if efp is not None:
             ax.plot(self.x, np.minimum(alpha, efp), 'k')
 
 
@@ -439,7 +440,7 @@ def three_classes_GMM_fit(x, test=None, alpha=0.01, prior_strength=100,
 
     nvox = np.size(x)
     x = np.reshape(x, (nvox, 1))
-    if test == None:
+    if test is None:
         test = x
     if np.size(test) == 0:
         return None
@@ -449,8 +450,8 @@ def three_classes_GMM_fit(x, test=None, alpha=0.01, prior_strength=100,
 
     # set the priors from a reasonable model of the data (!)
     # prior means
-    mb0 = np.mean(sx[ : alpha * nvox])
-    mb2 = np.mean(sx[(1 - alpha) * nvox:])
+    mb0 = np.mean(sx[:int(alpha * nvox)])
+    mb2 = np.mean(sx[int((1 - alpha) * nvox):])
     prior_means = np.reshape(np.array([mb0, 0, mb2]), (nclasses, 1))
     if fixed_scale:
         prior_scale = np.ones((nclasses, 1, 1)) * 1. / (prior_strength)
@@ -494,7 +495,7 @@ def three_classes_GMM_fit(x, test=None, alpha=0.01, prior_strength=100,
         return bfp, BayesianGMM
 
 
-def gamma_gaussian_fit(x, test=None, verbose=0, mpaxes=None,
+def gamma_gaussian_fit(x, test=None, verbose=0, mpaxes=False,
                        bias=1, gaussian_mix=0, return_estimator=False):
     """
     Computing some prior probabilities that the voxels of a certain map
@@ -510,9 +511,10 @@ def gamma_gaussian_fit(x, test=None, verbose=0, mpaxes=None,
     verbose: 0, 1 or 2, optional
         verbosity mode, 0 is quiet, and 2 calls matplotlib to display
         graphs.
-    mpaxes: matplotlib axes, option.
+    mpaxes: matplotlib axes, optional
         axes handle used to plot the figure in verbose mode
         if None, new axes are created
+        if false, nothing is done
     bias: float, optional
         lower bound on the Gaussian variance (to avoid shrinkage)
     gaussian_mix: float, optional
@@ -534,7 +536,7 @@ def gamma_gaussian_fit(x, test=None, verbose=0, mpaxes=None,
     Ggg = ggmixture.GGGM()
     Ggg.init_fdr(x)
     Ggg.estimate(x, niter=100, delta=1.e-8, bias=bias, verbose=0,
-                    gaussian_mix=gaussian_mix)
+                 gaussian_mix=gaussian_mix)
     if mpaxes is not False:
         # hyper-verbose mode
         Ggg.show(x, mpaxes=mpaxes)
@@ -551,16 +553,15 @@ def gamma_gaussian_fit(x, test=None, verbose=0, mpaxes=None,
 
 
 def smoothed_histogram_from_samples(x, bins=None, nbins=256, normalized=False):
-    """Returns the smooth histogram corresponding to the  density
-    underlying the samples in x
+    """ Smooth histogram corresponding to density underlying the samples in `x`
 
     Parameters
     ----------
-    x: array of shape(n_samples),
+    x: array of shape(n_samples)
        input data
-    bins: array of shape(nbins+1), optional,
+    bins: array of shape(nbins+1), optional
        the bins location
-    nbins: int, optional,
+    nbins: int, optional
        the number of bins of the resulting histogram
     normalized: bool, optional
        if True, the result is returned as a density value
