@@ -91,7 +91,7 @@ _b0*x1 + _b1*x3 + _b2*x1*x3 + _b3
 The I is the "intercept" term, I have explicity not used R's default of
 adding it to everything.
 
->>> f.design(r)
+>>> f.design(r)  #doctest: +STRUCTARR_EQUAL
 array([(51.0, 39.0, 1989.0, 1.0), (64.0, 54.0, 3456.0, 1.0),
        (70.0, 69.0, 4830.0, 1.0), (63.0, 47.0, 2961.0, 1.0),
        (78.0, 66.0, 5148.0, 1.0), (55.0, 44.0, 2420.0, 1.0),
@@ -109,25 +109,30 @@ array([(51.0, 39.0, 1989.0, 1.0), (64.0, 54.0, 3456.0, 1.0),
        (85.0, 71.0, 6035.0, 1.0), (82.0, 59.0, 4838.0, 1.0)], 
       dtype=[('x1', '<f8'), ('x3', '<f8'), ('x1*x3', '<f8'), ('1', '<f8')])
 '''
+from __future__ import print_function
+from __future__ import absolute_import
 
+import warnings
+import itertools
 from string import ascii_letters, digits
 
 import numpy as np
 from scipy.linalg import pinv
 
 import sympy
+from sympy import Dummy, default_sort_key
+from sympy.utilities.lambdify import (implemented_function, lambdify)
 
-from distutils.version import LooseVersion
-SYMPY_0p6 = LooseVersion(sympy.__version__) < LooseVersion('0.7.0')
-
+from nipy.utils import _NoValue, VisibleDeprecationWarning
 from nipy.utils.compat3 import to_str
 
-from nipy.fixes.sympy.utilities.lambdify import (implemented_function,
-                                                 lambdify)
-
 from nipy.algorithms.utils.matrices import matrix_rank, full_rank
 
+# Legacy repr printing from numpy.
+from nipy.testing import legacy_printing as setup_module  # noqa
 
+
+@np.deprecate(message = "Please use sympy.Dummy instead of this function")
 def make_dummy(name):
     """ Make dummy variable of given name
 
@@ -142,11 +147,11 @@ def make_dummy(name):
 
     Notes
     -----
-    The interface to Dummy changed between 0.6.7 and 0.7.0
+    The interface to Dummy changed between 0.6.7 and 0.7.0, and we used this
+    function to keep compatibility. Now we depend on sympy 0.7.0 and this
+    function is obsolete.
     """
-    if SYMPY_0p6:
-        return sympy.Symbol(name, dummy=True)
-    return sympy.Dummy(name)
+    return Dummy(name)
 
 
 def define(*args, **kwargs):
@@ -173,7 +178,7 @@ class Term(sympy.Symbol):
     >>> xval = np.array([(3,),(4,),(5,)], np.dtype([('x', np.float)]))
     >>> f = t.formula
     >>> d = f.design(xval)
-    >>> print d.dtype.descr
+    >>> print(d.dtype.descr)
     [('x', '<f8')]
     >>> f.design(xval, return_float=True)
     array([ 3.,  4.,  5.])
@@ -184,13 +189,13 @@ class Term(sympy.Symbol):
 
     def _getformula(self):
         return Formula([self])
-    formula = property(_getformula, doc="Return a Formula with only terms=[self].")
+    formula = property(_getformula,
+                       doc="Return a Formula with only terms=[self].")
 
     def __add__(self, other):
         if self == other:
             return self
-        else:
-            return sympy.Symbol.__add__(self, other)
+        return sympy.Symbol.__add__(self, other)
 
 
 # time symbol
@@ -228,13 +233,8 @@ def terms(names, **kwargs):
     >>> terms('abc')
     abc
     '''
-    if (SYMPY_0p6
-        and isinstance(names, basestring)
-        and not set(', ').intersection(names)):
-        if not kwargs.get('each_char', False):
-            # remove each_char (or no-op if absent)
-            kwargs.pop('each_char', None)
-            names = (names,)
+    if 'each_char' in kwargs:
+        raise TypeError('deprecated "each_char" kwarg removed in sympy>0.7.3')
     syms = sympy.symbols(names, **kwargs)
     try:
         len(syms)
@@ -293,7 +293,7 @@ def getparams(expression):
     >>> f.mean*sympy.exp(th)
     (_b0*x + _b1*y + _b2*z)*exp(theta)
     >>> getparams(f.mean*sympy.exp(th))
-    [theta, _b0, _b1, _b2]
+    [_b0, _b1, _b2, theta]
     """
     atoms = set([])
     expression = np.array(expression)
@@ -307,7 +307,7 @@ def getparams(expression):
     for atom in atoms:
         if isinstance(atom, sympy.Symbol) and not is_term(atom):
             params.append(atom)
-    params.sort()
+    params.sort(key=default_sort_key)
     return params
 
 
@@ -335,87 +335,123 @@ def getterms(expression):
     for atom in atoms:
         if is_term(atom):
             terms.append(atom)
-    terms.sort()
+    terms.sort(key=default_sort_key)
     return terms
 
 
-def make_recarray(rows, names, dtypes=None):
+def _recarray_from_array(arr, names, drop_name_dim=_NoValue):
+    """ Create recarray from input array `arr`, field names `names`
+    """
+    if not arr.dtype.isbuiltin:  # Structured array as input
+        # Rename fields
+        dtype = np.dtype([(n, d[1]) for n, d in zip(names, arr.dtype.descr)])
+        return arr.view(dtype)
+    # Can drop name axis for > 1D arrays or row vectors (scalar per name).
+    can_name_drop = arr.ndim > 1 or len(names) > 1
+    if can_name_drop and drop_name_dim is _NoValue:
+        warnings.warn(
+            'Default behavior of make_recarray and > 1D arrays will '
+            'change in next Nipy release.  Current default returns\n'
+            'array with same number of dimensions as input, with '
+            'axis corresponding to the field names having length 1\n; '
+            'Future default will be to drop this length 1 axis. Please '
+            'change your code to use explicit True or False for\n'
+            'compatibility with future Nipy.',
+            VisibleDeprecationWarning,
+            stacklevel=2)
+        # This default will change to True in next version of Nipy
+        drop_name_dim = False
+    dtype = np.dtype([(n, arr.dtype) for n in names])
+    # At least for numpy <= 1.7.1, the dimension that numpy applies the names
+    # to depends on the memory layout (C or F).  Ensure C layout for consistent
+    # application of names to last dimension.
+    rec_arr = np.ascontiguousarray(arr).view(dtype)
+    if can_name_drop and drop_name_dim:
+        rec_arr.shape = arr.shape[:-1]
+    return rec_arr
+
+
+def make_recarray(rows, names, dtypes=None, drop_name_dim=_NoValue):
     """ Create recarray from `rows` with field names `names`
 
-    Create a recarray with named columns from a list of rows and names
-    for the columns. If dtype is None, the dtype is based on rows if it
-    is an np.ndarray, else the data is cast as np.float. If dtypes are
-    supplied, it uses the dtypes to create a np.dtype unless rows is an
-    np.ndarray, in which case dtypes are ignored
+    Create a recarray with named columns from a list or ndarray of `rows` and
+    sequence of `names` for the columns. If `rows` is an ndarray, `dtypes` must
+    be None, otherwise we raise a ValueError. Otherwise, if `dtypes` is None,
+    we cast the data in all columns in `rows` as np.float. If `dtypes` is not
+    None, the routine uses `dtypes` as a dtype specifier for the output
+    structured array.
 
     Parameters
     ----------
-    rows: array-like
+    rows: list or array
         Rows that will be turned into an recarray.
     names: sequence
         Sequence of strings - names for the columns.
-    dtypes: [str or np.dtype]
-        Used to create a np.dtype, can be np.dtypes or string.
+    dtypes: None or sequence of str or sequence of np.dtype, optional
+        Used to create a np.dtype, can be sequence of np.dtype or string.
+    drop_name_dim : {_NoValue, False, True}, optional
+        Flag for compatibility with future default behavior.  Current default
+        is False.  If True, drops the length 1 dimension corresponding to the
+        axis transformed into fields when converting into a recarray.  If
+        _NoValue specified, gives default.  Default will change to True in the
+        next version of Nipy.
 
     Returns
     -------
     v : np.ndarray
+        Structured array with field names given by `names`.
 
     Examples
     --------
-    The following tests depend on machine byte order to pass
+    The following tests depend on machine byte order for their exact output.
 
-    >>> arr = np.array([[3,4 ], [4, 6], [6, 8]])
-    >>> make_recarray(arr, ['x', 'y']) #doctest: +ELLIPSIS
+    >>> arr = np.array([[3, 4], [4, 6], [6, 8]])
+    >>> make_recarray(arr, ['x', 'y'],
+    ...               drop_name_dim=True) #doctest: +STRUCTARR_EQUAL
+    array([(3, 4), (4, 6), (6, 8)],
+          dtype=[('x', '<i8'), ('y', '<i8')])
+    >>> make_recarray(arr, ['x', 'y'],
+    ...               drop_name_dim=False) #doctest: +STRUCTARR_EQUAL
     array([[(3, 4)],
            [(4, 6)],
            [(6, 8)]],
-          dtype=[('x', '...'), ('y', '...')])
-    >>> r = make_recarray(arr, ['w', 'u'])
-    >>> make_recarray(r, ['x', 'y']) #doctest: +ELLIPSIS
-    array([[(3, 4)],
-           [(4, 6)],
-           [(6, 8)]],
-          dtype=[('x', '...'), ('y', '...')])
-    >>> make_recarray([[3, 4], [4, 6], [7, 9]], 'wv', [np.float, np.int]) #doctest: +ELLIPSIS
+          dtype=[('x', '<i8'), ('y', '<i8')])
+    >>> r = make_recarray(arr, ['w', 'u'], drop_name_dim=True)
+    >>> make_recarray(r, ['x', 'y'],
+    ...               drop_name_dim=True) #doctest: +STRUCTARR_EQUAL
+    array([(3, 4), (4, 6), (6, 8)],
+          dtype=[('x', '<i8'), ('y', '<i8')])
+    >>> make_recarray([[3, 4], [4, 6], [7, 9]], 'wv',
+    ...               [np.float, np.int])  #doctest: +STRUCTARR_EQUAL
     array([(3.0, 4), (4.0, 6), (7.0, 9)],
-          dtype=[('w', '...'), ('v', '...')])
+          dtype=[('w', '<f8'), ('v', '<i8')])
+
+    Raises
+    ------
+    ValueError
+        `dtypes` not None when `rows` is array.
     """
     # XXX This function is sort of one of convenience
     # Would be nice to use DataArray or something like that
     # to add axis names.
     if isinstance(rows, np.ndarray):
-        if rows.dtype.isbuiltin:
-            dtype = np.dtype([(n, rows.dtype) for n in names])
-        else:
-            dtype = np.dtype([(n, d[1]) for n, d in zip(names, rows.dtype.descr)])
         if dtypes is not None:
             raise ValueError('dtypes not used if rows is an ndarray')
-        return rows.view(dtype)
+        return _recarray_from_array(rows, names, drop_name_dim)
+    # Structured array from list
     if dtypes is None:
         dtype = np.dtype([(n, np.float) for n in names])
     else:
         dtype = np.dtype([(n, d) for n, d in zip(names, dtypes)])
-    nrows = []
-    vector = -1
-    for r in rows:
-        if vector < 0:
-            a = np.array(r)
-            if a.shape == ():
-                vector = True
-            else:
-                vector = False
-        if not vector:
-            nrows.append(tuple(r))
-        else:
-            nrows.append(r)
-    if vector:
-        if len(names) != 1: # a 'row vector'
-            nrows = tuple(nrows)
-            return np.array(nrows, dtype)
-        else:
-            nrows = np.array([(r,) for r in nrows], dtype)
-    return np.array(nrows, dtype)
+    # Peek at first value in iterable
+    irows = iter(rows)
+    row0 = next(irows)
+    irows = itertools.chain([row0], irows)
+    if np.array(row0).shape == ():  # a vector
+        if len(names) != 1:  # a 'row vector'
+            return np.array(tuple(irows), dtype)
+        return np.array([(r,) for r in irows], dtype)
+    return np.array([tuple(r) for r in irows], dtype)
 
 
 class Formula(object):
@@ -425,10 +461,10 @@ class Formula(object):
     model being the sum of each term multiplied by a linear regression
     coefficient.
 
-    The expressions may depend on additional Symbol instances,
-    giving a non-linear regression model.
+    The expressions may depend on additional Symbol instances, giving a
+    non-linear regression model.
     """
-    # This flag is defined to avoid using isinstance
+    # This flag is defined for test isformula(obj) instead of isinstance
     _formula_flag = True
 
     def __init__(self, seq, char = 'b'):
@@ -477,7 +513,7 @@ class Formula(object):
                     "variables in front.")
 
     def _getdiff(self):
-        params = sorted(list(set(getparams(self.mean))))
+        params = sorted(list(set(getparams(self.mean))), key=default_sort_key)
         return [sympy.diff(self.mean, p).doit() for p in params]
     design_expr = property(_getdiff)
 
@@ -487,7 +523,7 @@ class Formula(object):
     dtype = property(_getdtype, doc='The dtype of the design matrix of the Formula.')
 
     def __repr__(self):
-        return """Formula(%s)""" % `list(self.terms)`
+        return "Formula(%r)" % (list(self.terms),)
 
     def __getitem__(self, key):
         """ Return the term such that str(term) == key.
@@ -535,7 +571,7 @@ class Formula(object):
         if keep:
             return np.sum([t for n, t in f.items() if n in keep])
         else:
-            return np.sum(f.values())
+            return np.sum(list(f.values()))
 
     def subs(self, old, new):
         """ Perform a sympy substitution on all terms in the Formula
@@ -567,50 +603,78 @@ class Formula(object):
         return self.__class__([term.subs(old, new) for term in self.terms])
 
     def __add__(self, other):
-        """
-        Create a new Formula by combining terms
-        of other with those of self.
+        """ New Formula combining terms of `self` with those of `other`.
 
+        Parameters
+        ----------
+        other : Formula instance
+            Object for which ``is_formula(other)`` is True
+
+        Returns
+        -------
+        added : Formula instance
+            Formula combining terms of `self` with terms of `other`
+
+        Examples
+        --------
         >>> x, y, z = [Term(l) for l in 'xyz']
         >>> f1 = Formula([x,y,z])
         >>> f2 = Formula([y])+I
         >>> f3=f1+f2
-        >>> sorted(f1.terms)
+        >>> sorted(f1.terms, key=default_sort_key)
         [x, y, z]
-        >>> sorted(f2.terms)
+        >>> sorted(f2.terms, key=default_sort_key)
         [1, y]
-        >>> sorted(f3.terms)
+        >>> sorted(f3.terms, key=default_sort_key)
         [1, x, y, y, z]
         """
         if not is_formula(other):
             raise ValueError('only Formula objects can be added to a Formula')
-        f = self.__class__(np.hstack([self.terms, other.terms]))
+        f = Formula(np.hstack([self.terms, other.terms]))
         return f
 
     def __sub__(self, other):
-        """
-        Create a new Formula by deleting terms in other
-        from self. No exceptions are raised for terms in other that do not appear in
-        self.
+        """ New Formula by deleting terms in `other` from those in `self`
 
+        Create and return a new Formula by deleting terms in `other` from those
+        in `self`.
+
+        No exceptions are raised for terms in `other` that do not appear in
+        `self`.
+
+        Parameters
+        ----------
+        other : Formula instance
+            Object for which ``is_formula(other)`` is True
+
+        Returns
+        -------
+        subbed : Formula instance
+            Formula with terms of `other` removed from terms of `self`
+
+        Examples
+        --------
         >>> x, y, z = [Term(l) for l in 'xyz']
-        >>> f1 = Formula([x,y,z])
-        >>> f2 = Formula([y])+I
+        >>> f1 = Formula([x, y, z])
+        >>> f2 = Formula([y]) + I
         >>> f1.mean
         _b0*x + _b1*y + _b2*z
         >>> f2.mean
         _b0*y + _b1
-        >>> f3=f2-f1
+        >>> f3 = f2 - f1
         >>> f3.mean
         _b0
-        >>> f4=f1-f2
+        >>> f4 = f1 - f2
         >>> f4.mean
         _b0*x + _b1*z
         """
         if not is_formula(other):
-            raise ValueError('only Formula objects can be subtracted from a Formula')
-        d = list(set(self.terms).difference(other.terms))
-        return self.__class__(d)
+            raise ValueError(
+                'only Formula objects can be subtracted from a Formula')
+        # Preserve order of terms in subtraction
+        unwanted = set(other.terms)
+        d = [term for term in self.terms if term not in unwanted]
+        return Formula(d)
 
     def __array__(self):
         return self.terms
@@ -636,7 +700,8 @@ class Formula(object):
                     v.append(Term.__mul__(oterm, sterm))
                 else:
                     v.append(sterm*oterm)
-        return Formula(tuple(np.unique(v)))
+        terms = sorted(set(v), key=default_sort_key)
+        return Formula(tuple(terms))
 
     def __eq__(self, other):
         s = np.array(self)
@@ -646,11 +711,11 @@ class Formula(object):
         return np.alltrue(np.equal(np.array(self), np.array(other)))
 
     def _setup_design(self):
-        """
-        Create a callable object to evaluate the design matrix
-        at a given set of parameter values to be specified by
-        a recarray and observed Term values, also specified
-        by a recarray.
+        """ Initialize design
+
+        Create a callable object to evaluate the design matrix at a given set
+        of parameter values to be specified by a recarray and observed Term
+        values, also specified by a recarray.
         """
         # the design expression is the differentiation of the expression
         # for the mean.  It is a list
@@ -673,7 +738,7 @@ class Formula(object):
 
         # This renaming is here principally because of the intercept.
 
-        random_offset = np.random.random_integers(low=0, high=2**30)
+        random_offset = np.random.randint(low=0, high=2**30)
 
         terms = getterms(self.mean)
 
@@ -692,7 +757,7 @@ class Formula(object):
         params = getparams(self.design_expr)
         newparams = []
         for i, p in enumerate(params):
-            newp = make_dummy("__p%d__" % (i + random_offset))
+            newp = Dummy("__p%d__" % (i + random_offset))
             for j, _ in enumerate(d):
                 d[j] = d[j].subs(p, newp)
             newparams.append(newp)
@@ -778,14 +843,14 @@ class Formula(object):
         # The input to design should have field names for all fields in self._dtypes['preterm']
         if not set(preterm_recarray.dtype.names).issuperset(self._dtypes['preterm'].names):
             raise ValueError("for term, expecting a recarray with "
-                             "dtype having the following names: %s"
-                             % `self._dtypes['preterm'].names`)
+                             "dtype having the following names: %r"
+                             % (self._dtypes['preterm'].names,))
         # The parameters should have field names for all fields in self._dtypes['param']
         if param_recarray is not None:
             if not set(param_recarray.dtype.names).issuperset(self._dtypes['param'].names):
                 raise ValueError("for param, expecting a recarray with "
-                                 "dtype having the following names: %s"
-                                 % `self._dtypes['param'].names`)
+                                 "dtype having the following names: %r"
+                                 % (self._dtypes['param'].names,))
         # If the only term is an intercept,
         # the return value is a matrix of 1's.
         if list(self.terms) == [sympy.Number(1)]:
@@ -921,7 +986,7 @@ def natural_spline(t, knots=None, order=3, intercept=False):
            [   5.,   25.,  125.,   64.,    8.,    1.],
            [   7.,   49.,  343.,  216.,   64.,   27.]])
     >>> d = n.design(xval)
-    >>> print d.dtype.descr
+    >>> print(d.dtype.descr)
     [('ns_1(x)', '<f8'), ('ns_2(x)', '<f8'), ('ns_3(x)', '<f8'), ('ns_4(x)', '<f8'), ('ns_5(x)', '<f8'), ('ns_6(x)', '<f8')]
     """
     if knots is None:
@@ -963,16 +1028,15 @@ class Factor(Formula):
     _factor_flag = True
 
     def __init__(self, name, levels, char='b'):
-        """
+        """ Initialize Factor
+
         Parameters
         ----------
         name : str
         levels : [str or int]
             A sequence of strings or ints.
-        char : str
-
-        Returns
-        -------
+        char : str, optional
+            prefix character for regression coefficients
         """
         # Check whether they can all be cast to strings or ints without
         # loss.
@@ -1013,9 +1077,9 @@ class Factor(Formula):
 
         Parameters
         ----------
-        variable : str or a simple sympy expression whose string representation
-            are all lower or upper case letters, i.e. it can be interpreted
-            as a name
+        variable : str or simple sympy expression
+            If sympy expression, then string representation must be all lower
+            or upper case letters, i.e. it can be interpreted as a name.
 
         Returns
         -------
@@ -1047,13 +1111,11 @@ class Factor(Formula):
         ----------
         col : ndarray
             an array with ndim==1
-
         name : str
             name of the Factor
 
         Returns
         -------
-
         factor : Factor
 
         Examples
@@ -1062,10 +1124,10 @@ class Factor(Formula):
         >>> f1 = Factor.fromcol(data['y'], 'y')
         >>> f2 = Factor.fromcol(data['x'], 'x')
         >>> d = f1.design(data)
-        >>> print d.dtype.descr
+        >>> print(d.dtype.descr)
         [('y_a', '<f8'), ('y_b', '<f8')]
         >>> d = f2.design(data)
-        >>> print d.dtype.descr
+        >>> print(d.dtype.descr)
         [('x_3', '<f8'), ('x_4', '<f8'), ('x_5', '<f8')]
         """
         col = np.asarray(col)
@@ -1197,13 +1259,13 @@ class RandomEffects(Formula):
 
         self._counter = 0
         if sigma is None:
-            self.sigma = np.diag([make_dummy('s2_%d' % i) for i in range(q)])
+            self.sigma = np.diag([Dummy('s2_%d' % i) for i in range(q)])
         else:
             self.sigma = sigma
         if self.sigma.shape != (q,q):
             raise ValueError('incorrect shape for covariance '
                              'of random effects, '
-                             'should have shape %s' % repr(q,q))
+                             'should have shape %r' % ((q,q)))
         self.char = char
 
     def cov(self, term, param=None):
