@@ -1,3 +1,8 @@
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
+
+from __future__ import print_function
+from __future__ import absolute_import
 
 from numpy.testing import assert_array_equal
 
@@ -22,5 +27,4 @@ def test_armodel():
             results = model.fit(y)
             rho, sigma = regression.yule_walker(y - results.predicted)
             model = regression.ARModel(model.design, rho)
-        print "AR coefficients:", model.rho
-
+        print("AR coefficients:", model.rho)
