@@ -1,13 +1,18 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+from __future__ import absolute_import
+
 import numpy as np
+
 from scipy.special import gammaln, hermitenorm
 import scipy.stats
 from scipy.misc import factorial
 
 from .. import rft
 
-from nipy.testing import assert_almost_equal, dec
+from nose.tools import assert_raises
+
+from numpy.testing import assert_almost_equal, dec
 
 #def rho(x, dim, df=np.inf):
 #    """
@@ -33,8 +38,17 @@ from nipy.testing import assert_almost_equal, dec
 #        else:
 #            return scipy.stats.norm.sf(x)
 
+def test_Q():
+    assert_raises(ValueError, rft.Q, -1)
+    assert_raises(ValueError, rft.Q, 0)
+    x = np.arange(-9, 10)
+    for dim in range(1, 4):
+        res = rft.Q(dim)
+        assert_almost_equal(res(x), hermitenorm(dim - 1)(x))
+
+
 def K(dim=4, dfn=7, dfd=np.inf):
-    """
+    r"""
     Determine the polynomial K in:
 
         Worsley, K.J. (1994). 'Local maxima and the expected Euler
@@ -95,7 +109,7 @@ def F(x, dim, dfd=np.inf, dfn=1):
 
 
 def polyF(dim, dfd=np.inf, dfn=1):
-    """
+    r"""
     Return the polynomial part of the EC density when evaluating the polynomial
     on the sqrt(F) scale (or sqrt(chi^2)=chi scale).
 
@@ -386,7 +400,7 @@ def test_hotelling2():
         chi = rft.ChiSquared(dfn=dfn)(x)
         assert_almost_equal(h, chi)
         chi2 = scipy.stats.chi2.sf(x, dfn)
-        yield assert_almost_equal, h, chi2
+        assert_almost_equal(h, chi2)
         # XXX - p appears to be unused
         p = rft.spherical_search(dfn)
         for dfd in [40,50]:
@@ -394,8 +408,8 @@ def test_hotelling2():
             h = rft.Hotelling(dfd=dfd,k=dfn)(x)
             f = scipy.stats.f.sf(x*fac, dfn, dfd-dfn+1)
             f2 = rft.FStat(dfd=dfd-dfn+1,dfn=dfn)(x*fac)
-            yield assert_almost_equal, f2, f
-            yield assert_almost_equal, h, f
+            assert_almost_equal(f2, f)
+            assert_almost_equal(h, f)
 
 
 @dec.slow
