@@ -1,30 +1,37 @@
+from __future__ import absolute_import
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+from itertools import combinations
+
 import numpy as np
+
 from scipy.stats import norm
-TINY = 1e-15
+
+TINY = 1e-16
 
 
 def z_score(pvalue):
     """ Return the z-score corresponding to a given p-value.
     """
-    pvalue = np.minimum(np.maximum(pvalue, TINY), 1. - TINY)
+    pvalue = np.minimum(np.maximum(pvalue, 1.e-300), 1. - TINY)
     z = norm.isf(pvalue)
     return z
 
 
 def multiple_fast_inv(a):
-    """Compute the inverse of a set of arrays.
+    """ Compute the inverse of a set of arrays in-place
 
     Parameters
     ----------
-    a: array_like of shape (n_samples, n_dim, n_dim)
-        Set of square matrices to be inverted. A is changed in place.
+    a: array_like of shape (n_samples, M, M)
+        Set of square matrices to be inverted. `a` is changed in place.
 
     Returns
     -------
-    a: ndarray
-       yielding the inverse of the inputs
+    a: ndarray shape (n_samples, M, M)
+       The input array `a`, overwritten with the inverses of the original 2D
+       arrays in ``a[0], a[1], ...``.  Thus ``a[0]`` replaced with
+       ``inv(a[0])`` etc.
 
     Raises
     ------
@@ -35,40 +42,47 @@ def multiple_fast_inv(a):
 
     Notes
     -----
-    This function is borrowed from scipy.linalg.inv, 
-    but with some customizations for speed-up.
+    This function is copied from scipy.linalg.inv, but with some customizations
+    for speed-up from operating on multiple arrays.  It also has some
+    conditionals to work with different scipy versions.
     """
-    if a.shape[1] != a.shape[2]:
-        raise ValueError('a must have shape(n_samples, n_dim, n_dim)')
-    from scipy.linalg import calc_lwork
-    from scipy.linalg.lapack import  get_lapack_funcs
-    a1, n = a[0], a.shape[0]
-    getrf, getri = get_lapack_funcs(('getrf', 'getri'), (a1,))
-    for i in range(n):
-        if (getrf.module_name[:7] == 'clapack'
-            and getri.module_name[:7] != 'clapack'):
-            # ATLAS 3.2.1 has getrf but not getri.
-            lu, piv, info = getrf(np.transpose(a[i]), rowmajor=0,
-                                  overwrite_a=True)
-            a[i] = np.transpose(lu)
-        else:
-            a[i], piv, info = getrf(a[i], overwrite_a=True)
+    # Consider errors for sparse, masked, object arrays, as for
+    # _asarray_validated?
+    from scipy.linalg.lapack import get_lapack_funcs
+    S, M, N = a.shape
+    if M != N:
+        raise ValueError('a must have shape(n_samples, M, M)')
+    a = np.asarray_chkfinite(a)
+    getrf, getri = get_lapack_funcs(('getrf','getri'), (a[0],))
+    # Calculate lwork on different scipy versions
+    try:
+        getri_lwork, = get_lapack_funcs(('getri_lwork',), (a[0],))
+    except (ValueError, AttributeError):  # scipy < 0.15
+        # scipy 0.10, 0.11 -> AttributeError
+        # scipy 0.12, 0.13, 0.14 -> ValueError
+        from scipy.linalg import calc_lwork
+        lwork = calc_lwork.getri(getri.prefix, M)[1]
+    else:  # scipies >= 0.15 have getri_lwork function
+        lwork, info = getri_lwork(M)
+        if info != 0:
+            raise ValueError('internal getri work space query failed: %d' % (info,))
+        lwork = int(lwork.real)
+    # XXX: the following line fixes curious SEGFAULT when
+    # benchmarking 500x500 matrix inverse. This seems to
+    # be a bug in LAPACK ?getri routine because if lwork is
+    # minimal (when using lwork[0] instead of lwork[1]) then
+    # all tests pass. Further investigation is required if
+    # more such SEGFAULTs occur.
+    lwork = int(1.01 * lwork)
+    for i, ai in enumerate(a):
+        lu, piv, info = getrf(ai, overwrite_a=True)
         if info == 0:
-            if getri.module_name[:7] == 'flapack':
-                lwork = calc_lwork.getri(getri.prefix, a1.shape[0])
-                lwork = lwork[1]
-                # XXX: the following line fixes curious SEGFAULT when
-                # benchmarking 500x500 matrix inverse. This seems to
-                # be a bug in LAPACK ?getri routine because if lwork is
-                # minimal (when using lwork[0] instead of lwork[1]) then
-                # all tests pass. Further investigation is required if
-                # more such SEGFAULTs occur.
-                lwork = int(1.01 * lwork)
-                a[i], _ = getri(a[i], piv, lwork=lwork, overwrite_lu=1)
-            else:  # clapack
-                a[i], _ = getri(a[i], piv, overwrite_lu=1)
-        else:
-            raise ValueError('Matrix LU decomposition failed')
+            a[i], info = getri(lu, piv, lwork=lwork, overwrite_lu=1)
+        if info > 0:
+            raise np.linalg.LinAlgError("singular matrix")
+        if info < 0:
+            raise ValueError('illegal value in %d-th argument of internal '
+                             'getrf|getri' % -info)
     return a
 
 
@@ -108,28 +122,6 @@ def multiple_mahalanobis(effect, covariance):
     return sqd
 
 
-# Taken from python doc site, exists in python2.6
-def combinations(iterable, r):
-    # combinations('ABCD', 2) --> AB AC AD BC BD CD
-    # combinations(range(4), 3) --> 012 013 023 123
-    pool = tuple(iterable)
-    n = len(pool)
-    if r > n:
-        return
-    indices = range(r)
-    yield tuple(pool[i] for i in indices)
-    while True:
-        for i in reversed(range(r)):
-            if indices[i] != i + n - r:
-                break
-        else:
-            return
-        indices[i] += 1
-        for j in range(i+1, r):
-            indices[j] = indices[j-1] + 1
-        yield tuple(pool[i] for i in indices)
-
-
 def complex(maximal=[(0, 3, 2, 7),
                      (0, 6, 2, 7),
                      (0, 7, 5, 4),
@@ -244,10 +236,11 @@ def join_complexes(*complexes):
         faces[i+1] = set([])
     for c in complexes:
         for i in range(nmax):
-            if c.has_key(i+1):
+            if i+1 in c:
                 faces[i+1] = faces[i+1].union(c[i+1])
     return faces
 
+
 def decompose3d(shape, dim=4):
     """
     Return all (dim-1)-dimensional simplices in a triangulation
@@ -273,7 +266,7 @@ def decompose3d(shape, dim=4):
     for i in range(4):
         unique[i+1] = c[i+1].difference(union[i+1])
 
-    if unique.has_key(dim) and dim > 1:
+    if dim in unique and dim > 1:
         d = unique[dim]
 
         for i in range(shape[0]-1):
@@ -301,7 +294,7 @@ def decompose3d(shape, dim=4):
         for i in range(3):
             unique[i+1] = c[i+1].difference(union[i+1])
         
-        if unique.has_key(dim) and dim > 1:
+        if dim in unique and dim > 1:
             d = unique[dim]
 
             for i in range(_shape[0]-1):
@@ -320,7 +313,7 @@ def decompose3d(shape, dim=4):
         for i in range(2):
             unique[i+1] = c[i+1].difference(union[i+1])
 
-        if unique.has_key(dim) and dim > 1:
+        if dim in unique and dim > 1:
             d = unique[dim]
 
             for i in range(_shape-1):
@@ -352,7 +345,7 @@ def decompose2d(shape, dim=3):
     for i in range(3):
         unique[i+1] = c[i+1].difference(union[i+1])
 
-    if unique.has_key(dim) and dim > 1:
+    if dim in unique and dim > 1:
         d = unique[dim]
 
         for i in range(shape[0]-1):
@@ -371,7 +364,7 @@ def decompose2d(shape, dim=3):
         for i in range(2):
             unique[i+1] = c[i+1].difference(union[i+1])
 
-        if unique.has_key(dim) and dim > 1:
+        if dim in unique and dim > 1:
             d = unique[dim]
 
             for i in range(_shape-1):
@@ -425,3 +418,25 @@ def test_EC2(shape):
 test_EC2.__test__ = False
 
 
+def check_cast_bin8(arr):
+    """ Return binary array `arr` as uint8 type, or raise if not binary.
+
+    Parameters
+    ----------
+    arr : array-like
+
+    Returns
+    -------
+    bin8_arr : uint8 array
+        `bin8_arr` has same shape as `arr`, is of dtype ``np.uint8``, with
+        values 0 and 1 only.
+
+    Raises
+    ------
+    ValueError
+        When the array is not binary.  Speficically, raise if, for any element
+        ``e``, ``e != (e != 0)``.
+    """
+    if np.any(arr != (arr !=0)):
+        raise ValueError('input array should only contain values 0 and 1')
+    return arr.astype(np.uint8)
