@@ -4,15 +4,13 @@
 This module describes two types of *mappings*:
 
 * CoordinateMap: a general function from a domain to a range, with a possible
-     inverse function.
-
+  inverse function;
 * AffineTransform: an affine function from a domain to a range, not
-     necessarily of the same dimension, hence not always invertible.
+  necessarily of the same dimension, hence not always invertible.
 
-Each of these objects is meant to encapsulate a tuple of
-(domain, range, function).
-Each of the mapping objects contain all the details about their domain
-CoordinateSystem, their range CoordinateSystem and the mapping between
+Each of these objects is meant to encapsulate a tuple of (domain, range,
+function).  Each of the mapping objects contain all the details about their
+domain CoordinateSystem, their range CoordinateSystem and the mapping between
 them.
 
 Common API
@@ -21,13 +19,16 @@ Common API
 They are separate classes, neither one inheriting from the other.
 They do, however, share some parts of an API, each having methods:
 
-* renamed_domain : rename on the coordinates of the domain (returns a new mapping)
+* renamed_domain : rename on the coordinates of the domain (returns a new
+  mapping)
 
 * renamed_range : rename the coordinates of the range (returns a new mapping)
 
-* reordered_domain : reorder the coordinates of the domain (returns a new mapping)
+* reordered_domain : reorder the coordinates of the domain (returns a new
+  mapping)
 
-* reordered_range : reorder the coordinates of the range (returns a new mapping)
+* reordered_range : reorder the coordinates of the range (returns a new
+  mapping)
 
 * inverse : when appropriate, return the inverse *mapping*
 
@@ -45,21 +46,21 @@ Operations on mappings (module level functions)
 -----------------------------------------------
 
 * compose : Take a sequence of mappings (either CoordinateMaps or
-   AffineTransforms) and return their composition. If they are all
-   AffineTransforms, an AffineTransform is returned. This checks to
-   ensure that domains and ranges of the various mappings agree.
+  AffineTransforms) and return their composition. If they are all
+  AffineTransforms, an AffineTransform is returned. This checks to ensure that
+  domains and ranges of the various mappings agree.
 * product : Take a sequence of mappings (either CoordinateMaps or
-   AffineTransforms) and return a new mapping that has domain and range
-   given by the concatenation of their domains and ranges, and the
-   mapping simply concatenates the output of each of the individual
-   mappings. If they are all AffineTransforms, an AffineTransform is
-   returned. If they are all AffineTransforms that are in fact linear
-   (i.e. origin=0) then can is represented as a block matrix with the
-   size of the blocks determined by
+  AffineTransforms) and return a new mapping that has domain and range given by
+  the concatenation of their domains and ranges, and the mapping simply
+  concatenates the output of each of the individual mappings. If they are all
+  AffineTransforms, an AffineTransform is returned. If they are all
+  AffineTransforms that are in fact linear (i.e. origin=0) then can is
+  represented as a block matrix with the size of the blocks determined by
 * concat : Take a mapping and prepend a coordinate to its domain and
-   range.  For mapping `m`, this is the same as
-   product(AffineTransform.identity('concat'), `m`)
+  range.  For mapping ``m``, this is the same as
+  ``product(AffineTransform.identity('concat'), m)``
 """
+from __future__ import absolute_import
 
 import warnings
 
@@ -75,6 +76,9 @@ from .coordinate_system import(CoordinateSystem,
                                product as coordsys_product
                                )
 
+# Legacy repr printing from numpy.
+from nipy.testing import legacy_printing as setup_module  # noqa
+
 # shorthand
 CS = CoordinateSystem
 
@@ -640,7 +644,7 @@ class AffineTransform(object):
             return None
         except TypeError:
             # Try using sympy for the inverse.  This might be needed for sympy
-            # symbols in the affine, or Float128
+            # symbols in the affine, or Float128, or complex numbers.
             from sympy import Matrix, matrix2numpy
             sym_inv = Matrix(self.affine).inv()
             m_inv = matrix2numpy(sym_inv).astype(aff_dt)
@@ -1194,7 +1198,7 @@ def reordered_domain(mapping, order=None):
     """
     ndim = mapping.ndims[0]
     if order is None:
-        order = range(ndim)[::-1]
+        order = list(range(ndim))[::-1]
     elif type(order[0]) == type(''):
         order = [mapping.function_domain.index(s) for s in order]
 
@@ -1365,13 +1369,13 @@ def renamed_domain(mapping, newnames, name=''):
        ...
     ValueError: no domain coordinate named l
     """
-    for key in newnames.keys():
+    for key in list(newnames):
         if type(key) == type(0):
             newnames[mapping.function_domain.coord_names[key]] = \
                 newnames[key]
             del(newnames[key])
 
-    for key in newnames.keys():
+    for key in list(newnames):
         if key not in mapping.function_domain.coord_names:
             raise ValueError('no domain coordinate named %s' % str(key))
 
@@ -1429,13 +1433,13 @@ def renamed_range(mapping, newnames):
        ...
     ValueError: no range coordinate named w
     """
-    for key in newnames.keys():
+    for key in list(newnames):
         if type(key) == type(0):
             newnames[mapping.function_range.coord_names[key]] = \
                 newnames[key]
             del(newnames[key])
 
-    for key in newnames.keys():
+    for key in list(newnames):
         if key not in mapping.function_range.coord_names:
             raise ValueError('no range coordinate named %s' % str(key))
 
@@ -1500,7 +1504,7 @@ def reordered_range(mapping, order=None):
     """
     ndim = mapping.ndims[1]
     if order is None:
-        order = range(ndim)[::-1]
+        order = list(range(ndim))[::-1]
     elif type(order[0]) == type(''):
         order = [mapping.function_range.index(s) for s in order]
 
@@ -1823,14 +1827,14 @@ def drop_io_dim(cm, axis_id, fix0=True):
             raise AxisError('Input and output dimensions not orthogonal to '
                             'rest of affine')
     M, N = aff.shape
-    rows = range(M)
-    cols = range(N)
+    rows = list(range(M))
+    cols = list(range(N))
     in_dims = list(cm.function_domain.coord_names)
     out_dims = list(cm.function_range.coord_names)
-    if not in_dim is None:
+    if in_dim is not None:
         in_dims.pop(in_dim)
         cols.pop(in_dim)
-    if not out_dim is None:
+    if out_dim is not None:
         out_dims.pop(out_dim)
         rows.pop(out_dim)
     aff = aff[rows]
