@@ -15,10 +15,15 @@ they are applied on the left of the vector.  For example:
 >>> vec = np.array([1, 2, 3]).reshape((3,1)) # column vector
 >>> tvec = np.dot(M, vec)
 '''
+from __future__ import absolute_import
 
 import math
+
 import numpy as np
 
+# Legacy repr printing from numpy.
+from nipy.testing import legacy_printing as setup_module  # noqa
+
 _MAX_FLOAT = np.maximum_sctype(np.float)
 _FLOAT_EPS = np.finfo(np.float).eps
 
