@@ -1,10 +1,11 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+from __future__ import absolute_import, print_function
+
 import time
 
 import numpy as np
 
-from ....testing import *
 from .. import copy_vector
 
 
