@@ -4,6 +4,9 @@
 Utilities for extracting masks from EPI images and applying them to time
 series.
 """
+from __future__ import absolute_import
+
+import math
 
 # Major scientific libraries imports
 import numpy as np
@@ -11,8 +14,9 @@ from scipy import ndimage
 
 # Neuroimaging libraries imports
 from nibabel import load, nifti1, save
-from nibabel.loadsave import read_img_data
 
+from ..io.nibcompat import get_header, get_affine, get_unscaled_data
+from ..externals.six import string_types
 
 ###############################################################################
 # Operating on connect component
@@ -38,7 +42,7 @@ def largest_cc(mask):
         raise ValueError('No non-zero values: no connected components')
     if label_nb == 1:
         return mask.astype(np.bool)
-    label_count = np.bincount(labels.ravel())
+    label_count = np.bincount(labels.ravel().astype(np.int))
     # discard 0 the 0 label
     label_count[0] = 0
     return labels == label_count.argmax()
@@ -122,12 +126,12 @@ def compute_mask_files(input_filename, output_filename=None,
         The main of all the images used to estimate the mask. Only
         provided if `return_mean` is True.
     """
-    if isinstance(input_filename, basestring):
+    if isinstance(input_filename, string_types):
         # One single filename or image
         nim = load(input_filename)  # load the image from the path
-        vol_arr = read_img_data(nim, prefer='unscaled')
-        header = nim.get_header()
-        affine = nim.get_affine()
+        vol_arr = get_unscaled_data(nim)
+        header = get_header(nim)
+        affine = get_affine(nim)
         if vol_arr.ndim == 4:
             if isinstance(vol_arr, np.memmap):
                 # Get rid of memmapping: it is faster.
@@ -157,8 +161,8 @@ def compute_mask_files(input_filename, output_filename=None,
             if index == 0:
                 first_volume = nim.get_data().squeeze()
                 mean_volume = first_volume.copy().astype(np.float32)
-                header = nim.get_header()
-                affine = nim.get_affine()
+                header = get_header(nim)
+                affine = get_affine(nim)
             else:
                 mean_volume += nim.get_data().squeeze()
         mean_volume /= float(len(list(input_filename)))
@@ -227,8 +231,8 @@ def compute_mask(mean_volume, reference_volume=None, m=0.2, M=0.9,
     sorted_input = np.sort(mean_volume.reshape(-1))
     if exclude_zeros:
         sorted_input = sorted_input[sorted_input != 0]
-    limiteinf = np.floor(m * len(sorted_input))
-    limitesup = np.floor(M * len(sorted_input))
+    limiteinf = int(math.floor(m * len(sorted_input)))
+    limitesup = int(math.floor(M * len(sorted_input)))
 
     delta = sorted_input[limiteinf + 1:limitesup + 1] \
             - sorted_input[limiteinf:limitesup]
@@ -367,7 +371,7 @@ def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
     threshold = min(threshold, 1 - 1.e-7)
 
     for this_mask in input_masks:
-        if isinstance(this_mask, basestring):
+        if isinstance(this_mask, string_types):
             # We have a filename
             this_mask = load(this_mask).get_data()
         if grp_mask is None:
@@ -385,10 +389,10 @@ def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
         grp_mask = largest_cc(grp_mask)
 
     if output_filename is not None:
-        if isinstance(input_masks[0], basestring):
+        if isinstance(input_masks[0], string_types):
             nim = load(input_masks[0])
-            header = nim.get_header()
-            affine = nim.get_affine()
+            header = get_header(nim)
+            affine = get_affine(nim)
         else:
             header = dict()
             affine = np.eye(4)
@@ -442,16 +446,16 @@ def series_from_mask(filenames, mask, dtype=np.float32,
     if smooth:
         # Convert from a sigma to a FWHM:
         smooth /= np.sqrt(8 * np.log(2))
-    if isinstance(filenames, basestring):
+    if isinstance(filenames, string_types):
         # We have a 4D nifti file
         data_file = load(filenames)
-        header = data_file.get_header()
+        header = get_header(data_file)
         series = data_file.get_data()
         if ensure_finite:
             # SPM tends to put NaNs in the data outside the brain
             series[np.logical_not(np.isfinite(series))] = 0
         series = series.astype(dtype)
-        affine = data_file.get_affine()[:3, :3]
+        affine = get_affine(data_file)[:3, :3]
         del data_file
         if isinstance(series, np.memmap):
             series = np.asarray(series).copy()
@@ -473,7 +477,7 @@ def series_from_mask(filenames, mask, dtype=np.float32,
                 data[np.logical_not(np.isfinite(data))] = 0
             data = data.astype(dtype)
             if smooth is not False:
-                affine = data_file.get_affine()[:3, :3]
+                affine = get_affine(data_file)[:3, :3]
                 vox_size = np.sqrt(np.sum(affine ** 2, axis=0))
                 smooth_sigma = smooth / vox_size
                 data = ndimage.gaussian_filter(data, smooth_sigma)
@@ -482,6 +486,6 @@ def series_from_mask(filenames, mask, dtype=np.float32,
             # Free memory early
             del data
             if index == 0:
-                header = data_file.get_header()
+                header = get_header(data_file)
 
     return series, header
