@@ -3,11 +3,11 @@
 """
 TODO
 """
+from __future__ import absolute_import
 
 __docformat__ = 'restructuredtext'
 
-import fmri, hrf, utils, formula
-import fmristat
+from . import fmristat
 
 from nipy.testing import Tester
 test = Tester().test
