@@ -1,6 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-from __future__ import with_statement
 """
 This module implements fMRI Design Matrix creation.
 
@@ -23,6 +22,8 @@ Design matrices contain three different types of regressors:
 
 Author: Bertrand Thirion, 2009-2011
 """
+from __future__ import absolute_import
+
 import numpy as np
 
 from warnings import warn
@@ -61,30 +62,40 @@ def _poly_drift(order, frametimes):
     return pol
 
 
-def _cosine_drift(hfcut, frametimes):
-    """Create a cosine drift matrix
+def _cosine_drift(period_cut, frametimes):
+    """Create a cosine drift matrix with periods greater or equals to period_cut
 
     Parameters
     ----------
-    hfcut, float , cut frequency of the low-pass filter
-    frametimes: array of shape(nscans): the sampling time
+    period_cut: float 
+         Cut period of the low-pass filter (in sec)
+    frametimes: array of shape(nscans)
+         The sampling times (in sec)
 
     Returns
     -------
     cdrift:  array of shape(n_scans, n_drifts)
-             polynomial drifts plus a constant regressor
+             cosin drifts plus a constant regressor at cdrift[:,0]
+
+    Ref: http://en.wikipedia.org/wiki/Discrete_cosine_transform DCT-II
     """
-    tmax = float(frametimes.max())
-    tsteps = len(frametimes)
-    order = int(np.floor(2 * float(tmax) / float(hfcut)) + 1)
-    cdrift = np.zeros((tsteps, order))
+    len_tim = len(frametimes)
+    n_times = np.arange(len_tim)
+    hfcut = 1./ period_cut # input parameter is the period  
+
+    dt = frametimes[1] - frametimes[0] # frametimes.max() should be (len_tim-1)*dt    
+    order = int(np.floor(2*len_tim*hfcut*dt)) # s.t. hfcut = 1/(2*dt) yields len_tim
+    cdrift = np.zeros((len_tim, order))
+    nfct = np.sqrt(2.0/len_tim)
+    
     for k in range(1, order):
-        cdrift[:, k - 1] = np.sqrt(2.0 / tmax) * np.cos(
-            np.pi * (frametimes / tmax + 0.5 / tsteps) * k)
-    cdrift[:, order - 1] = np.ones_like(frametimes)
+        cdrift[:,k-1] = nfct * np.cos((np.pi/len_tim)*(n_times + .5)*k)
+    
+    cdrift[:,order-1] = 1. # or 1./sqrt(len_tim) to normalize
     return cdrift
 
 
+
 def _blank_drift(frametimes):
     """ Create the blank drift matrix
 
@@ -131,7 +142,8 @@ def _make_drift(drift_model, frametimes, order=1, hfcut=128.):
     return drift, names
 
 
-def _convolve_regressors(paradigm, hrf_model, frametimes, fir_delays=[0]):
+def _convolve_regressors(paradigm, hrf_model, frametimes, fir_delays=[0],
+                         min_onset=-24):
     """ Creation of  a matrix that comprises
     the convolution of the conditions onset with a certain hrf model
 
@@ -146,6 +158,9 @@ def _convolve_regressors(paradigm, hrf_model, frametimes, fir_delays=[0]):
     fir_delays=[0], optional, array of shape(nb_onsets) or list
                     in case of FIR design, yields the array of delays
                     used in the FIR model
+    min_onset: float, optional
+        minimal onset relative to frametimes[0] (in seconds)
+        events that start before frametimes[0] + min_onset are not considered
 
     Returns
     -------
@@ -179,11 +194,12 @@ def _convolve_regressors(paradigm, hrf_model, frametimes, fir_delays=[0]):
         else:
             duration = paradigm.duration[paradigm.con_id == nc]
         exp_condition = (onsets, duration, values)
-        reg, names = compute_regressor(exp_condition, hrf_model, frametimes,
-                                       con_id=nc, fir_delays=fir_delays,
-                                       oversampling=oversampling)
+        reg, names = compute_regressor(
+            exp_condition, hrf_model, frametimes, con_id=nc,
+            fir_delays=fir_delays, oversampling=oversampling,
+            min_onset=min_onset)
         hnames += names
-        if rmatrix == None:
+        if rmatrix is None:
             rmatrix = reg
         else:
             rmatrix = np.hstack((rmatrix, reg))
@@ -224,16 +240,17 @@ def _full_rank(X, cmax=1e15):
 
 class DesignMatrix():
     """ This is a container for a light-weight class for design matrices
-    This class is only used to make IO and visualization
 
-    Class members
-    -------------
-    matrix: array of shape(n_scans, n_regressors),
-            the numerical specification of the matrix
-    names: list of len (n_regressors);
-           the names associated with the columns
-    frametimes: array of shape(n_scans), optional,
-                the occurrence time of the matrix rows
+    This class is only used to make IO and visualization.
+
+    Attributes
+    ----------
+    matrix: array of shape (n_scans, n_regressors)
+        the numerical specification of the matrix.
+    names: list of len (n_regressors)
+        the names associated with the columns.
+    frametimes: array of shape (n_scans), optional
+        the occurrence time of the matrix rows.
     """
 
     def __init__(self, matrix, names, frametimes=None):
@@ -249,7 +266,7 @@ class DesignMatrix():
                     'The number %d of frametimes is different from the' + \
                     'number %d of rows' % (frametimes.size, matrix.shape[0]))
 
-        self.frametimes = frametimes
+        self.frametimes = np.asarray(frametimes, dtype=np.float)
         self.matrix = matrix_
         self.names = names
 
@@ -270,15 +287,17 @@ class DesignMatrix():
             writer.writerow(self.names)
             writer.writerows(self.matrix)
 
-    def show(self, rescale=True, ax=None):
+    def show(self, rescale=True, ax=None, cmap=None):
         """Visualization of a design matrix
 
         Parameters
         ----------
         rescale: bool, optional
-                 rescale columns magnitude for visualization or not
+                 rescale columns magnitude for visualization or not.
         ax: axis handle, optional
-            Handle to axis onto which we will draw design matrix
+            Handle to axis onto which we will draw design matrix.
+        cmap: colormap, optional
+            Matplotlib colormap to use, passed to `imshow`.
 
         Returns
         -------
@@ -294,10 +313,49 @@ class DesignMatrix():
             plt.figure()
             ax = plt.subplot(1, 1, 1)
 
-        ax.imshow(x, interpolation='Nearest', aspect='auto')
+        ax.imshow(x, interpolation='Nearest', aspect='auto',
+                  cmap=cmap)
         ax.set_label('conditions')
         ax.set_ylabel('scan number')
 
+        if self.names is not None:
+            ax.set_xticks(list(range(len(self.names))))
+            ax.set_xticklabels(self.names, rotation=60, ha='right')
+        return ax
+
+    def show_contrast(self, contrast, ax=None, cmap=None):
+        """
+        Plot a contrast for a design matrix.
+
+        Parameters
+        ----------
+        contrast : np.float
+            Array forming contrast with respect to the design matrix.
+        ax: axis handle, optional
+            Handle to axis onto which we will draw design matrix.
+        cmap: colormap, optional
+            Matplotlib colormap to use, passed to `imshow`.
+
+        Returns
+        -------
+        ax: axis handle
+        """
+        import matplotlib.pyplot as plt
+
+        contrast = np.atleast_2d(contrast)
+
+        # normalize the values per column for better visualization
+        if ax is None:
+            plt.figure()
+            ax = plt.subplot(1, 1, 1)
+
+        ax.imshow(contrast, interpolation='Nearest', 
+                  aspect='auto',
+                  cmap=cmap)
+        ax.set_label('conditions')
+        ax.set_yticks(range(contrast.shape[0]))
+        ax.set_yticklabels([])
+
         if self.names is not None:
             ax.set_xticks(range(len(self.names)))
             ax.set_xticklabels(self.names, rotation=60, ha='right')
@@ -305,8 +363,8 @@ class DesignMatrix():
 
 
 def make_dmtx(frametimes, paradigm=None, hrf_model='canonical',
-              drift_model='cosine', hfcut=128, drift_order=1,
-              fir_delays=[0], add_regs=None, add_reg_names=None):
+              drift_model='cosine', hfcut=128, drift_order=1, fir_delays=[0],
+              add_regs=None, add_reg_names=None, min_onset=-24):
     """ Generate a design matrix from the input parameters
 
     Parameters
@@ -315,13 +373,14 @@ def make_dmtx(frametimes, paradigm=None, hrf_model='canonical',
     paradigm: Paradigm instance, optional
               description of the experimental paradigm
     hrf_model: string, optional,
-               that specifies the hemodynamic response function
-               it can be 'canonical', 'canonical with derivative' or 'fir'
+               that specifies the hemodynamic response function.
+               Can be one of {'canonical', 'canonical with derivative',
+               'fir', 'spm', 'spm_time', 'spm_time_dispersion'}.
     drift_model: string, optional
                  specifies the desired drift model,
                  to be chosen among 'polynomial', 'cosine', 'blank'
     hfcut: float, optional
-           cut frequency of the low-pass filter
+           cut period of the low-pass filter
     drift_order: int, optional
                  order of the drift model (in case it is polynomial)
     fir_delays: array of shape(nb_onsets) or list, optional,
@@ -332,12 +391,17 @@ def make_dmtx(frametimes, paradigm=None, hrf_model='canonical',
     add_reg_names: list of (naddreg) regressor names, optional
                    if None, while naddreg>0, these will be termed
                    'reg_%i',i=0..naddreg-1
+    min_onset: float, optional
+        minimal onset relative to frametimes[0] (in seconds)
+        events that start before frametimes[0] + min_onset are not considered
 
     Returns
     -------
     DesignMatrix instance
     """
     # check arguments
+    frametimes = np.asarray(frametimes, dtype=np.float)
+
     # check that additional regressor specification is correct
     n_add_regs = 0
     if add_regs is not None:
@@ -351,7 +415,7 @@ def make_dmtx(frametimes, paradigm=None, hrf_model='canonical',
             'time-frames: %s' % (add_regs.shape[0], np.size(frametimes)))
 
     # check that additional regressor names are well specified
-    if  add_reg_names == None:
+    if  add_reg_names is None:
         add_reg_names = ['reg%d' % k for k in range(n_add_regs)]
     elif len(add_reg_names) != n_add_regs:
         raise ValueError(
@@ -367,7 +431,7 @@ def make_dmtx(frametimes, paradigm=None, hrf_model='canonical',
     if paradigm is not None:
         # create the condition-related regressors
         matrix, names = _convolve_regressors(
-            paradigm, hrf_model.lower(), frametimes, fir_delays)
+            paradigm, hrf_model.lower(), frametimes, fir_delays, min_onset)
 
     # step 2: additional regressors
     if add_regs is not None:
@@ -418,7 +482,7 @@ def dmtx_from_csv(path, frametimes=None):
 
 def dmtx_light(frametimes, paradigm=None, hrf_model='canonical',
                drift_model='cosine', hfcut=128, drift_order=1, fir_delays=[0],
-               add_regs=None, add_reg_names=None, path=None):
+               add_regs=None, add_reg_names=None, min_onset=-24, path=None):
     """Make a design matrix while avoiding framework
 
     Parameters
@@ -434,7 +498,8 @@ def dmtx_light(frametimes, paradigm=None, hrf_model='canonical',
         the names of the columns of the design matrix
     """
     dmtx_ = make_dmtx(frametimes, paradigm, hrf_model, drift_model, hfcut,
-                      drift_order, fir_delays, add_regs, add_reg_names)
+                      drift_order, fir_delays, add_regs, add_reg_names,
+                      min_onset)
     if path is not None:
         dmtx_.write_csv(path)
     return dmtx_.matrix, dmtx_.names
