@@ -1,17 +1,20 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-import gc, os
-from tempfile import mkstemp
+from __future__ import with_statement
+from __future__ import absolute_import
+
+import gc
 import warnings
 
 import numpy as np
 
 from nipy.modalities.fmri.api import axis0_generator, FmriImageList
-from nipy.core.api import parcels, fromarray
+from nipy.core.api import parcels, Image, AffineTransform as AfT
 from nipy.io.api import  load_image, save_image
 
 from nose.tools import assert_equal, assert_true
 
+from nibabel.tmpdirs import InTemporaryDirectory
 from nipy.testing import funcfile
 
 
@@ -26,23 +29,19 @@ def teardown():
 
 
 def test_write():
-    fp, fname = mkstemp('.nii')
+    fname = 'myfile.nii'
     img = load_image(funcfile)
-    save_image(img, fname)
-    test = FmriImageList.from_image(load_image(fname))
-    yield assert_equal, test[0].affine.shape, (4,4)
-    yield assert_equal, img[0].affine.shape, (5,4)
-
-    # Check the affine...
-    A = np.identity(4)
-    A[:3,:3] = img[:,:,:,0].affine[:3,:3]
-    A[:3,-1] = img[:,:,:,0].affine[:3,-1]
-    yield assert_true, np.allclose(test[0].affine, A)
-
-    # Under windows, if you don't close before delete, you get a
-    # locking error.
-    os.close(fp)
-    os.remove(fname)
+    with InTemporaryDirectory():
+        save_image(img, fname)
+        test = FmriImageList.from_image(load_image(fname))
+        assert_equal(test[0].affine.shape, (4,4))
+        assert_equal(img[0].affine.shape, (5,4))
+        # Check the affine...
+        A = np.identity(4)
+        A[:3,:3] = img[:,:,:,0].affine[:3,:3]
+        A[:3,-1] = img[:,:,:,0].affine[:3,-1]
+        assert_true(np.allclose(test[0].affine, A))
+        del test
 
 
 def test_iter():
@@ -68,8 +67,8 @@ def test_subcoordmap():
 def test_labels1():
     img = load_image(funcfile)
     data = img.get_data()
-    parcelmap = fromarray(np.asarray(img[0]), 'kji', 'zyx')
-    parcelmap = (np.asarray(parcelmap) * 100).astype(np.int32)
+    parcelmap = Image(img[0].get_data(), AfT('kji', 'zyx', np.eye(4)))
+    parcelmap = (parcelmap.get_data() * 100).astype(np.int32)
     v = 0
     for i, d in axis0_generator(data, parcels(parcelmap)):
         v += d.shape[1]
