@@ -22,10 +22,11 @@ Examples
 (17, 21, 3, 20)
 
 """
+from __future__ import absolute_import
 
 import os
 
-#__all__ = ['funcfile', 'anatfile']
+import numpy as np
 
 # Discover directory path
 filepath = os.path.abspath(__file__)
@@ -45,6 +46,15 @@ from . import decorators as dec
 # Allow failed import of nose if not now running tests
 try:
     from nose.tools import assert_true, assert_false
-    from lightunit import ParametricTestCase, parametric
 except ImportError:
     pass
+
+
+def legacy_printing():
+    """ Set numpy print options to "legacy" for new versions of numpy
+
+    If imported into a file, nosetest will run this before any doctests.
+    """
+    from distutils.version import LooseVersion
+    if LooseVersion(np.__version__) >= LooseVersion('1.14'):
+        np.set_printoptions(legacy="1.13")
